home · contact · privacy
Allow selection of grid geometry via MAP command. A lot refactoring.
authorChristian Heller <c.heller@plomlompom.de>
Fri, 18 Jan 2019 01:02:26 +0000 (02:02 +0100)
committerChristian Heller <c.heller@plomlompom.de>
Fri, 18 Jan 2019 01:02:26 +0000 (02:02 +0100)
client.py
game_common.py
server.py
server_/game.py

index 6594fb506fe7f78a8c050a5ce364dbbca9bd0ba6..e270a8c9a63d6dc424ce28ef3c91c38febbf387c 100755 (executable)
--- a/server.py
+++ b/server.py
@@ -20,7 +20,7 @@ if os.path.exists(game_file_name):
             print("FILE INPUT LINE %s: %s" % (i, line), end='')
             game.io.handle_input(line, store=False)
 else:
-    game.io.handle_input('MAP Y:5,X:5')
+    game.io.handle_input('MAP Hex Y:5,X:5')
     game.io.handle_input('TERRAIN_LINE 0 "xxxxx"')
     game.io.handle_input('TERRAIN_LINE 1 "x...x"')
     game.io.handle_input('TERRAIN_LINE 2 "x.X.x"')
index bcf41b7ddeb10bd50f3283da04b76c898bb00c7a..143f3199f9dfa417cdece5419619978b26eaf73f 100644 (file)
@@ -33,10 +33,6 @@ class Map(game_common.Map):
     #        for x in range(self.size[1]):
     #            yield ([y, x], self.terrain[self.get_position_index([y, x])])
 
-    @property
-    def size_i(self):
-        return self.size[0] * self.size[1]
-
     def get_directions(self):
         directions = []
         for name in dir(self):
@@ -45,21 +41,11 @@ class Map(game_common.Map):
         return directions
 
     def new_from_shape(self, init_char):
-        return Map(self.size, init_char*self.size_i)
-
-    #def are_neighbors(self, pos_1, pos_2):
-    #    return abs(pos_1[0] - pos_2[0]) <= 1 and abs(pos_1[1] - pos_2[1] <= 1)
-
-    def are_neighbors(self, pos_1, pos_2):
-        if pos_1[0] == pos_2[0] and abs(pos_1[1] - pos_2[1]) <= 1:
-            return True
-        elif abs(pos_1[0] - pos_2[0]) == 1:
-            if pos_1[0] % 2 == 0:
-                if pos_2[1] in (pos_1[1], pos_1[1] - 1):
-                    return True
-            elif pos_2[1] in (pos_1[1], pos_1[1] + 1):
-                return True
-        return False
+        import copy
+        new_map = copy.deepcopy(self)
+        for pos in new_map:
+            new_map[pos] = init_char
+        return new_map
 
     def move(self, start_pos, direction):
         mover = getattr(self, 'move_' + direction)
@@ -75,11 +61,19 @@ class Map(game_common.Map):
     def move_RIGHT(self, start_pos):
         return [start_pos[0], start_pos[1] + 1]
 
-    #def move_UP(self, start_pos):
-    #    return [start_pos[0] - 1, start_pos[1]]
 
-    #def move_DOWN(self, start_pos):
-    #    return [start_pos[0] + 1, start_pos[1]]
+class MapHex(Map):
+
+    def are_neighbors(self, pos_1, pos_2):
+        if pos_1[0] == pos_2[0] and abs(pos_1[1] - pos_2[1]) <= 1:
+            return True
+        elif abs(pos_1[0] - pos_2[0]) == 1:
+            if pos_1[0] % 2 == 0:
+                if pos_2[1] in (pos_1[1], pos_1[1] - 1):
+                    return True
+            elif pos_2[1] in (pos_1[1], pos_1[1] + 1):
+                return True
+        return False
 
     def move_UPLEFT(self, start_pos):
         if start_pos[0] % 2 == 0:
@@ -106,14 +100,27 @@ class Map(game_common.Map):
             return [start_pos[0] + 1, start_pos[1] + 1]
 
 
+class MapSquare(Map):
+
+    def are_neighbors(self, pos_1, pos_2):
+        return abs(pos_1[0] - pos_2[0]) <= 1 and abs(pos_1[1] - pos_2[1] <= 1)
+
+    def move_UP(self, start_pos):
+        return [start_pos[0] - 1, start_pos[1]]
+
+    def move_DOWN(self, start_pos):
+        return [start_pos[0] + 1, start_pos[1]]
+
+
 class World(game_common.World):
 
     def __init__(self):
         super().__init__()
-        self.Thing = Thing  # use local Thing class instead of game_common's
-        self.Map = Map # use local Map class instead of game_common's
-        self.map_ = Map()  # use extended child class
         self.player_id = 0
+        # use extended local classes
+        self.Thing = Thing
+        self.MapHex = MapHex
+        self.MapSquare = MapSquare
 
     def proceed_to_next_player_turn(self):
         """Run game world turns until player can decide their next step.
@@ -277,7 +284,8 @@ class Game(game_common.CommonCommandsMixin):
             return 'Y:' + str(tuple_[0]) + ',X:' + str(tuple_[1])
 
         self.io.send('NEW_TURN ' + str(self.world.turn))
-        self.io.send('MAP ' + stringify_yx(self.world.map_.size))
+        grid = self.world.map_.__class__.__name__[3:]
+        self.io.send('MAP ' + grid +' ' + stringify_yx(self.world.map_.size))
         visible_map = self.world.get_player().get_visible_map()
         for y, line in visible_map.lines():
             self.io.send('VISIBLE_MAP_LINE %5s %s' % (y, self.io.quote(line)))
--- a/client.py
+++ b/client.py
@@ -4,10 +4,55 @@ import plom_socket_io
 import socket
 import threading
 from parser import ArgError, Parser
-from game_common import World, CommonCommandsMixin
+import game_common
 
 
-class Game(CommonCommandsMixin):
+class MapSquare(game_common.Map):
+
+    def list_terrain_to_lines(self, terrain_as_list):
+        terrain = ''.join(terrain_as_list)
+        map_lines = []
+        start_cut = 0
+        while start_cut < len(terrain):
+            limit = start_cut + self.game.world.map_.size[1]
+            map_lines += [terrain[start_cut:limit]]
+            start_cut = limit
+        return "\n".join(map_lines)
+
+
+class MapHex(game_common.Map):
+
+    def list_terrain_to_lines(self, terrain_as_list):
+        new_terrain_list = []
+        x = 0
+        y = 0
+        for c in terrain_as_list:
+            new_terrain_list += [c, ' ']
+            x += 1
+            if x == self.size[1]:
+                new_terrain_list += ['\n']
+                x = 0
+                y += 1
+                if y % 2 != 0:
+                    new_terrain_list += [' ']
+        return ''.join(new_terrain_list)
+
+
+class World(game_common.World):
+
+    def __init__(self, *args, **kwargs):
+        """Extend original with local classes and empty default map.
+
+        We need the empty default map because we draw the map widget
+        on any update, even before we actually receive map data.
+        """
+        super().__init__(*args, **kwargs)
+        self.MapHex = MapHex
+        self.MapSquare = MapSquare
+        self.map_ = self.MapHex()
+
+
+class Game(game_common.CommonCommandsMixin):
     world = World()
     log_text = ''
 
@@ -64,29 +109,7 @@ class WidgetManager:
         for t in self.game.world.things:
             pos_i = self.game.world.map_.get_position_index(t.position)
             terrain_as_list[pos_i] = self.game.symbol_for_type(t.type_)
-
-        #terrain = ''.join(terrain_as_list)
-        #map_lines = []
-        #start_cut = 0
-        #while start_cut < len(terrain):
-        #    limit = start_cut + self.game.world.map_.size[1]
-        #    map_lines += [terrain[start_cut:limit]]
-        #    start_cut = limit
-        #return "\n".join(map_lines)
-
-        new_terrain_list = []
-        x = 0
-        y = 0
-        for c in terrain_as_list:
-            new_terrain_list += [c, ' ']
-            x += 1
-            if x == self.game.world.map_.size[1]:
-                new_terrain_list += ['\n']
-                x = 0
-                y += 1
-                if y % 2 != 0:
-                    new_terrain_list += [' ']
-        return ''.join(new_terrain_list)
+        return self.game.world.map_.list_terrain_to_lines(terrain_as_list)
 
     def update(self):
         """Redraw all non-edit widgets."""
index 5aa4b2544729d337fcef2ee939fe1ddeddc3ea63..b37e4475ffaf3fd420e53cd39eea6551c169bce2 100644 (file)
@@ -3,9 +3,13 @@ from parser import ArgError
 
 class Map:
 
-    def __init__(self, size=(0, 0), terrain=''):
+    def __init__(self, size=(0, 0)):
         self.size = size
-        self.terrain = terrain
+        self.terrain = '?'*self.size_i
+
+    @property
+    def size_i(self):
+        return self.size[0] * self.size[1]
 
     def set_line(self, y, line):
         height_map = self.size[0]
@@ -25,10 +29,8 @@ class Map:
 class World:
 
     def __init__(self):
-        self.Map = Map  # child classes may use an extended Map class here
         self.Thing = Thing  # child classes may use an extended Thing class here
         self.turn = 0
-        self.map_ = self.Map()
         self.things = []
 
     def get_thing(self, id_):
@@ -39,8 +41,9 @@ class World:
         self.things += [t]
         return t
 
-    def new_map(self, yx):
-        self.map_ = self.Map(yx, '?')
+    def new_map(self, geometry, yx):
+        map_type = getattr(self, 'Map' + geometry)
+        self.map_ = map_type(yx)
 
 
 class Thing:
@@ -54,10 +57,14 @@ class Thing:
 
 class CommonCommandsMixin:
 
-    def cmd_MAP(self, yx):
-        """Create new map of size yx and only '?' cells."""
-        self.world.new_map(yx)
-    cmd_MAP.argtypes = 'yx_tuple:nonneg'
+    def cmd_MAP(self, geometry, yx):
+        """Create new map of grid geometry, size yx and only '?' cells."""
+        legal_grids = {'Hex', 'Square'}
+        if geometry not in legal_grids:
+            raise ArgError('First map argument must be one of: ' +
+                           ', '.join(legal_grids))
+        self.world.new_map(geometry, yx)
+    cmd_MAP.argtypes = 'string yx_tuple:nonneg'
 
     def cmd_THING_TYPE(self, i, type_):
         t = self.world.get_thing(i)
index 2a7b55cf530439623c022e7db0c9dadc7b250b95..9791d5f0bacf830fd4faa19a2b23b6184c2be0b1 100755 (executable)